@echo off

rem Check that the installation path was provided
if "%~1" == "" (
  echo The syntax is:
  echo:
  echo %~n0 installation_path [compiler]
  echo:
  echo Description:
  echo 	installation_path:	Path where TELEMAC-MASCARET dependencies will be installed.
  echo 	compiler (optional^):	Compiler that should be used, either mingw (default^) or intel.
  exit /b 0
)

rem Create it is it does not exist
if not exist "%~1" (
  mkdir "%~1" || exit /b 0
)

rem Create the installation path
set install_path=%~1
mkdir %install_path% 2>nul
rem Keep the installation path using forward slashes for CMake (they are mandatory because ScaLAPACK is using a bugged CMake module)
set install_path=%install_path:\=/%

if "%~2" == "" (
  set compiler=mingw
) else (
  set compiler=%~2
)

rem Store compiler name
for /f "tokens=2 delims=:" %%a in ('"find "" ":%compiler:~,1%" 2>&1"') do set compiler_name=%%a%compiler:~1%

rem Check that Git is available
set git_path=
for /f "delims=" %%a in ('"where git 2>nul"') do set git_path=%%a
if not defined git_path (
  echo Error: Git not found. Please ensure that Git is in your PATH.
  exit /b 0
)

rem Check that CMake is available
set cmake_path=
for /f "delims=,tokens=*" %%a in ('"where cmake 2>nul"') do set cmake_path=%%a
if not defined cmake_path (
  echo Error: CMake not found. Please ensure that CMake is in your PATH.
  exit /b 0
)

rem ***************************
rem * Set dependency versions *
rem ***************************
set hdf5=hdf5-1.10.8
set med=med-4.1.0
set metis=metis-5.1.1
set msmpi=msmpi-10.1.2
set lapack=lapack-3.10.0
set scalapack=scalapack-2.1.0
set mumps=mumps-5.4.1
set aed2=libaed2-1.3.0
set gotm=gotm-2019-06-14-opentelemac
set python=python-3.7.9
rem GDAL, Fiona and Rasterio Python cannot be installed using PyPI on Windows
rem Also, the version of Shapely that comes through pip is unstable
rem Instead, we need to use unofficial wheels from Christoph Gohlke’s website
set gdal=GDAL-3.4.1-cp37-cp37m-win_amd64.whl
set fiona=Fiona-1.8.20-cp37-cp37m-win_amd64.whl
set rasterio=rasterio-1.2.10-cp37-cp37m-win_amd64.whl
set shapely=Shapely-1.8.0-cp37-cp37m-win_amd64.whl

pushd %~dp0deps

if %compiler% == mingw (
  goto mingw_build
)

if %compiler% == intel (
  goto intel_build
)

echo Unsupported compiler: %~1

:exit
popd
color 7
exit /b 0

:mingw_build
rem Try to find Mingw in the current PATH
set mingw_path=
for /f "delims=" %%a in ('"where mingw32-make 2>nul"') do set mingw_path=%%a
rem If not found, add the usual Mingw location to PATH
if not defined mingw_path set path=C:\mingw64\bin;%path%
for /f "delims=" %%a in ('"where mingw32-make 2>nul"') do set mingw_path=%%a
if not defined mingw_path (
  echo Error: Mingw not found. Please ensure that Mingw bin folder is in your PATH.
  goto exit
)

call :build_hdf5 || goto exit
call :build_med || goto exit
call :build_metis || goto exit
call :build_msmpi || goto exit
call :build_lapack || goto exit
call :build_scalapack || goto exit
call :build_mumps || goto exit
call :build_aed2 || goto exit
call :build_gotm || goto exit
call :install_python || goto exit
goto exit

:intel_build
call :build_hdf5 || goto exit
call :build_med || goto exit
call :build_metis || goto exit
rem call :build_mumps || goto exit
rem call :build_aed2 || goto exit
rem call :buid_gotm || goto exit
goto exit

:build_hdf5
rem ********
rem * HDF5 *
rem ********
call :download %hdf5% "https://www.hdfgroup.org/package/hdf5-1-10-8-zip/?wpdmdl=16060&refresh=61dee6661eab21641997926" || exit /b 1
set cmake_config_options=-DHDF5_GENERATE_HEADERS:BOOL=OFF
call :cmake_build %hdf5% || exit /b 1
exit /b 0

:build_med
rem *******
rem * MED *
rem *******
call :download %med% "https://files.salome-platform.org/Salome/other/med-4.1.0.tar.gz" || exit /b 1
set cmake_config_options=-DHDF5_ROOT_DIR:PATH=%install_path%/%hdf5%
if %compiler% == intel (
  rem MED libraries cannot be built as shared libraries with Visual Studio generator as they
  rem miss the proper __declspec(dllexport) declarations on each class and function
  set cmake_config_options=%cmake_config_options% -DMEDFILE_BUILD_SHARED_LIBS:BOOL=OFF -DMEDFILE_BUILD_STATIC_LIBS:BOOL=ON
)
call :cmake_build %med% || exit /b 1
exit /b 0

:build_metis
rem *********
rem * METIS *
rem *********
call :download %metis% "https://github.com/KarypisLab/METIS/archive/refs/tags/v5.1.1-DistDGL-v0.5.zip" || exit /b 1
pushd %metis%
rmdir GKLib 2>nul
call :download GKLib "https://github.com/KarypisLab/GKlib/archive/refs/tags/METIS-v5.1.1-DistDGL-0.5.zip" || popd && exit /b 1
popd
set cmake_config_options=-DBUILD_SHARED_LIBS:BOOL=ON
call :cmake_build %metis% || exit /b 1
exit /b 0

:build_msmpi
rem **********
rem * MS-MPI *
rem **********
echo Building %msmpi% with %compiler_name% compiler...
set carch=x86_64
pushd %msmpi%
cd sdk
rd /s/q build-%carch% 2>nul
mkdir build-%carch%
cd build-%carch%
mkdir bin include lib
rem Generate import library
dlltool -k -d ..\msmpi.def.%carch% -l lib\libmsmpi.a || popd && exit /b 1
rem Headers
copy ..\mpi.h include\
copy ..\mpif.h include\
copy ..\mpi.f90 include\
copy ..\mpifptr.h.%carch% include\mpifptr.h
rem Build compiler wrappers
set mpic=..\mpi.c
set cflags=-s -O2 -DNDEBUG
gcc %cflags% -o bin\mpicc.exe -DCC %mpic% || popd && exit /b 1
gcc %cflags% -o bin\mpicxx.exe -DCXX %mpic% || popd && exit /b 1
copy bin\mpicxx.exe "bin\mpic++.exe"
gcc %cflags% -o bin\mpifort.exe -DFC %mpic% || popd && exit /b 1
copy bin\mpifort.exe bin\mpif77.exe
copy bin\mpifort.exe bin\mpif90.exe
rem Build FORTRAN90 MPI modules
bin\mpifort -c -Jinclude ..\mpi.f90 || popd && exit /b 1
del mpi.o
cd ..\..
rem Installation
xcopy /e/y bin %install_path:/=\%\%msmpi%\bin\
xcopy /e/y license %install_path:/=\%\%msmpi%\license\
xcopy /e/y sdk\build-x86_64 %install_path:/=\%\%msmpi%\
popd
exit /b 0

:build_lapack
rem **********
rem * LAPACK *
rem **********
call :download %lapack% "https://github.com/Reference-LAPACK/lapack/archive/refs/tags/v3.10.0.zip" || exit /b 1
set cmake_config_options=-DCMAKE_BUILD_TYPE=Release
call :cmake_build %lapack% || exit /b 1
exit /b 0

:build_scalapack
rem *************
rem * ScaLAPACK *
rem *************
call :download %scalapack% "https://github.com/Reference-ScaLAPACK/scalapack/archive/refs/tags/v2.1.0.zip" || exit /b 1
rem We need to use a foward slash for lapack and blas libraries paths since CMake CheckFortranFunctionExists is bugged when using a backward slash
set cmake_config_options=-DMPI_BASE_DIR:PATH=%install_path%/%msmpi% -DLAPACK_LIBRARIES:STRING=%install_path%/%lapack%/lib/liblapack.a -DBLAS_LIBRARIES:STRING=%install_path%/%lapack%/lib/libblas.a
call :cmake_build %scalapack% || exit /b 1
exit /b 0

:build_mumps
rem *********
rem * MUMPS *
rem *********
call :download %mumps% "https://github.com/scivision/mumps/archive/refs/tags/v5.4.1.9.zip" || exit /b 1
set cmake_config_options=-DMUMPS_UPSTREAM_VERSION=5.4.1 -Dautobuild:BOOL=OFF -DLAPACK_ROOT:PATH=%install_path%/%lapack% -DSCALAPACK_ROOT:PATH=%install_path%/%scalapack%
if %compiler% == mingw (
  set cmake_config_options=%cmake_config_options% -DMPI_ROOT:PATH=%install_path%/%msmpi%
) else (
  if not defined MKLROOT (
    rem Set Intel MKL environment
    call "%ONEAPI_ROOT%mkl\latest\env\vars.bat"
  )
  if not defined I_MPI_ROOT (
    rem Set Intel MPI environment
    call "%I_MPI_ONEAPI_ROOT%\env\vars.bat"
  )
  set cmake_config_options=%cmake_config_options% -DMPI_ROOT:PATH="%I_MPI_ROOT%"
)
call :cmake_build %mumps% || exit /b 1
exit /b 0

:build_aed2
rem ********
rem * AED2 *
rem ********
call :download %aed2% "https://github.com/AquaticEcoDynamics/libaed2/archive/refs/tags/v1.3.0.zip" || exit /b 1
echo Building libaed2 with %compiler_name% compiler...
set install_dir=%install_path:/=\%\%aed2%
rd /s/q %install_dir% 2>nul
pushd %aed2%
mingw32-make || goto build_aed2_error
mkdir %install_dir%
xcopy /e/y include %install_dir%\include\
xcopy /e/y mod %install_dir%\include\
xcopy /e/y lib %install_dir%\lib\
popd
exit /b 0

:build_aed2_error
popd
exit /b 1

:build_gotm
rem ********
rem * GOTM *
rem ********
call :download %gotm% "http://www.opentelemac.org/index.php/component/jdownloads/finish/39-manual-installation-sources/2127-gotm/0?Itemid=54" || exit /b 1
set cmake_config_options=-DBUILD_SHARED_LIBS:BOOL=ON -DGOTM_USE_NetCDF=OFF
call :cmake_build %gotm% || exit /b 1
exit /b 0

:install_python
if exist %install_path%\%python% (
  exit /b 0
)

pushd %install_path:/=\%

echo Downloading and extracting %python%...
curl -L https://www.python.org/ftp/python/3.7.9/python-3.7.9-embed-amd64.zip -o python.zip || goto install_error
mkdir %python%
tar -xvzf python.zip -C %python% || goto install_error
del python.zip
rem Remove the _pth file
del %python%\python3*._pth

rem Install Python SDK to be able to use f2py
xcopy /e/y %~dp0pythonsdk\include %cd%\%python%\include\
xcopy /e/y %~dp0pythonsdk\libs %cd%\%python%\libs\

rem Download and install pip
curl -O -J https://bootstrap.pypa.io/get-pip.py || goto install_error
set path=%cd%\%python%\Scripts;%cd%\%python%;%path%
python get-pip.py
del get-pip.py

rem Install all Python requirements for TELEMAC-MASCARET
echo Installing Python requirements...
pip install -r %~dp0requirements.txt || goto install_error

rem Download GDAL, Fiona, Rasterio and Shapely unofficial wheels, to avoid building them locally
echo Downloading GDAL, Fiona and Rasterio...
curl -O -J https://download.lfd.uci.edu/pythonlibs/x6hvwk7i/%gdal% || goto install_error
curl -O -J https://download.lfd.uci.edu/pythonlibs/x6hvwk7i/%fiona% || goto install_error
curl -O -J https://download.lfd.uci.edu/pythonlibs/x6hvwk7i/%rasterio% || goto install_error
curl -O -J https://download.lfd.uci.edu/pythonlibs/x6hvwk7i/%shapely% || goto install_error
pip install %gdal% && del %gdal%
pip install %fiona% && del %fiona%
pip install %rasterio% && del %rasterio%
pip install %shapely% && del %shapely%
pip install geopandas

popd
exit /b 0

:install_error
popd
exit /b 1


:download
rem ******************************************
rem * Download a library if not already done *
rem ******************************************
set lib=%~1
set url="%~2"

if exist %lib% (
  exit /b 0
)

rem Download and extract
echo Downloading and extracting %lib%...
set output=%lib%_arch
curl -L %url% -o %output% || exit /b 1
mkdir %lib%
tar -xvzf %output% --strip-components=1 -C %lib% || exit /b 1
del %output%
rem Apply patch if needed
if exist %~dp0patches\%lib%.patch (
  echo Patching %lib%...
  pushd %lib%
  git apply --ignore-space-change %~dp0patches\%lib%.patch || popd && exit /b 1
  popd
)

exit /b 0

:cmake_build
rem ************************
rem * CMake build function *
rem ************************
set lib=%~1

if %compiler% == mingw (
  set cmake_config_options=%cmake_config_options% -G "MinGW Makefiles" -DCMAKE_BUILD_TYPE:STRING=Release
  set cmake_build_options=-j 4
) else (
  if %compiler% == intel (
    set cmake_config_options=%cmake_config_options% -G "Visual Studio 16 2019" -A x64 -T "Intel C++ Compiler 19.2" -DCMAKE_CONFIGURATION_TYPES:STRING=Release
    set cmake_build_options=--config Release -j 4
  ) else (
    echo Unknown compiler: %compiler%.
    exit /b 1
  )
)

echo Building %lib% with %compiler_name% compiler...

pushd %lib%
mkdir build-%compiler% 2>nul
cd build-%compiler%
cmake %cmake_config_options% -DCMAKE_INSTALL_PREFIX:PATH=%install_path%/%lib% .. || goto cmake_build_error
cmake --build . %cmake_build_options% || goto cmake_build_error
cmake --install . || goto cmake_build_error
popd
color 7
exit /b 0

:cmake_build_error
popd
exit /b 1
